@@ -23,7 +23,6 @@ $Id$
 __docformat__ = 'reStructuredText'
 
 import persistent.dict
-import rwproperty
 from decimal import Decimal
 
 import zope.interface
@@ -222,18 +221,8 @@ class GenericWorksheet(requirement.Requirement):
     hidden = False
 
 
-class Worksheet(GenericWorksheet):
-    zope.interface.implements(interfaces.IActivityWorksheet,
-                              annotation.interfaces.IAttributeAnnotatable)
-
-    def values(self):
-        activities = []
-        for activity in super(Worksheet, self).values():
-            if interfaces.ILinkedColumnActivity.providedBy(activity):
-                if isHiddenSource(activity.source):
-                    continue
-            activities.append(activity)
-        return activities
+class WorksheetAnnotatableMixin(object):
+    zope.interface.implements(annotation.interfaces.IAttributeAnnotatable)
 
     def getCategoryWeights(self):
         ann = annotation.interfaces.IAnnotations(self)
@@ -247,21 +236,25 @@ class Worksheet(GenericWorksheet):
             ann[CATEGORY_WEIGHTS_KEY] = persistent.dict.PersistentDict()
         ann[CATEGORY_WEIGHTS_KEY][category] = weight
 
+
+class Worksheet(GenericWorksheet, WorksheetAnnotatableMixin):
+    zope.interface.implements(interfaces.IActivityWorksheet,
+                              annotation.interfaces.IAttributeAnnotatable)
+
+    def values(self):
+        activities = []
+        for activity in super(Worksheet, self).values():
+            if interfaces.ILinkedColumnActivity.providedBy(activity):
+                if isHiddenSource(activity.source):
+                    continue
+            activities.append(activity)
+        return activities
+
     def canAverage(self):
-        if not self.deployed:
-            return True
-
-        section = ISection(self, None)
-        if section is None:
-            return False
-        courses = list(section.courses)
-        if not courses:
-            return False
-        sheets = interfaces.ICourseDeployedWorksheets(courses[0])
-        return self.__name__ in sheets
+        return True
 
 
-class ReportWorksheet(requirement.Requirement):
+class ReportWorksheet(requirement.Requirement, WorksheetAnnotatableMixin):
     zope.interface.implements(interfaces.IReportWorksheet)
 
     deployed = False
@@ -362,11 +355,11 @@ class LinkedActivity(Activity):
         self.source = external_activity.source
         self.external_activity_id = external_activity.external_activity_id
 
-    @rwproperty.getproperty
+    @property
     def points(self):
         return int(self.scoresystem.max)
 
-    @rwproperty.setproperty
+    @points.setter
     def points(self, value):
         self.scoresystem.max = Decimal(value)
 
